import sympy as sp # pip install sympy
from sympy.parsing.sympy_parser import parse_expr, standard_transformations, implicit_multiplication_application
from sympy.abc import _clash
import re
import warnings

warnings.filterwarnings("ignore", category=FutureWarning)

transformations = standard_transformations + (implicit_multiplication_application,)
local_dict = dict(_clash)

# Additional symbols
x, y, z, a, b, c, n, m, r, t, theta = sp.symbols('x y z a b c n m r t theta')

# Add custom dictionary for functions and constants
local_dict.update({
    'sin': sp.sin,
    'cos': sp.cos,
    'tan': sp.tan,
    'cot': sp.cot,
    'sec': sp.sec,
    'csc': sp.csc,
    'log': sp.log,
    'ln': sp.ln,
    'e': sp.E,
    'pi': sp.pi,
    'sqrt': sp.sqrt,
    'diff': sp.diff,
    'integrate': sp.integrate,
    'limit': sp.limit,
    'det': sp.det,
    'Matrix': sp.Matrix,
    'solve': sp.solve,
    'factor': sp.factor,
    'expand': sp.expand
})

def handle_equations(input_str):
    equations = input_str.split(" and ")
    parsed_eqs = []
    symbols_set = set()
    for eq in equations:
        lhs, rhs = eq.split("=")
        lhs = parse_expr(lhs, transformations=transformations, local_dict=local_dict)
        rhs = parse_expr(rhs, transformations=transformations, local_dict=local_dict)
        parsed_eqs.append(sp.Eq(lhs, rhs))
        symbols_set |= lhs.free_symbols | rhs.free_symbols
    solution = sp.solve(parsed_eqs, tuple(symbols_set), dict=True)
    return f"Solution: {solution}"

def handle_matrix(expr):
    matrix = parse_expr(expr, transformations=transformations, local_dict=local_dict)
    if isinstance(matrix, sp.Matrix):
        det = matrix.det()
        inv = matrix.inv() if matrix.det() != 0 else "Not Invertible"
        return f"Matrix: {matrix}\nDeterminant: {det}\nInverse: {inv}"
    return "Not a valid matrix expression."

def solve_expression(expr):
    try:
        if " and " in expr and "=" in expr:
            return handle_equations(expr)

        if expr.lower().startswith("matrix"):
            return handle_matrix(expr.split("=", 1)[-1].strip())

        parsed = parse_expr(expr, transformations=transformations, local_dict=local_dict, evaluate=False)
        simplified = sp.simplify(parsed)
        numeric = simplified.evalf()

        return f"Simplified: {simplified}\nNumeric: {numeric}"

    except Exception as e:
        return f"[ERROR] {str(e)}"

def MathAI(message: str = '1X + 2X - 3X'):
    expr = message.strip()
    result = solve_expression(expr)
    return result




import sys
from pyai import *
from pyai import Camera
from pyai import TTS
from flask import Flask, request, jsonify
import tkinter as tk
import customtkinter as ctk
from tkinter import filedialog, Menu, simpledialog, Toplevel
from PIL import Image, ImageTk, ImageOps
import os
import webbrowser

ctk.set_appearance_mode("dark")
ctk.set_default_color_theme("dark-blue")


class GUI(ctk.CTk):
    def __init__(self):
        super().__init__()

        self.title("AI Chat App with Image & Text Features")
        self.geometry("600x700")

        self.image_refs = []  # Prevent garbage collection
        self.loaded_images = []  # Store (PIL image, path)

        # Chat area
        self.chat_frame = ctk.CTkScrollableFrame(self, width=550, height=550)
        self.chat_frame.pack(padx=10, pady=10, fill="both", expand=True)

        # Input area
        self.entry_frame = ctk.CTkFrame(self)
        self.entry_frame.pack(padx=10, pady=(0, 10), fill="x")

        self.entry = ctk.CTkEntry(self.entry_frame, width=400, placeholder_text="Type your message here...")
        self.entry.grid(row=0, column=0, padx=(10, 5), pady=10, sticky="ew")
        self.entry.bind("<Return>", lambda e: self.send_message())

        self.send_button = ctk.CTkButton(self.entry_frame, text="Send", command=self.send_message)
        self.send_button.grid(row=0, column=1, padx=5)

        self.image_button = ctk.CTkButton(self.entry_frame, text="📷 Image", command=self.upload_image)
        self.image_button.grid(row=0, column=2, padx=5)

        self.entry_frame.columnconfigure(0, weight=1)

    def send_message(self):
        message = self.entry.get().strip()
        if message:
            self.display_message("You", message)
            self.entry.delete(0, "end")
            self.generate_ai_reply(message)

    def display_message(self, sender, message):
        full_message = f"{sender}: {message}"
        label = ctk.CTkLabel(self.chat_frame, text=full_message, anchor="w", justify="left", wraplength=500)
        label.pack(anchor="w", padx=10, pady=4)
        label.bind("<Button-3>", lambda e: self.show_text_menu(e, label, full_message))

    def generate_ai_reply(self, message):
        response = ""
        brain = AdvanceBrain(r'.\intents.json')  # Assuming AdvanceBrain is defined in pyai
        msg_type = brain.predict_message_type(message)
        if msg_type == ['Question', 'Answer']:
            response = brain.process_messages(message)

        if response == 'CLICK_PHOTO':
            root = tk.Tk()
            Camera(root)
            root.mainloop()

        elif response == 'TTS':
            TTS(message)

        elif response[0] == "OPEN":
            webbrowser.open(response[1])
        else:

            label = ctk.CTkLabel(self.chat_frame, text=response, anchor="e", justify="left", wraplength=500)
            label.pack(anchor="e", padx=10, pady=4)
            label.bind("<Button-3>", lambda e: self.show_text_menu(e, label, response))

    def upload_image(self):
        filepath = filedialog.askopenfilename(
            filetypes=[("Image files", "*.png *.jpg *.jpeg *.bmp *.gif")]
        )
        if filepath:
            img = Image.open(filepath)
            self.display_image(img, filepath)

    def display_image(self, image_obj, path=None):
        # Resize and convert to CTkImage
        img_resized = image_obj.resize((300, 300))
        ctk_img = ctk.CTkImage(light_image=img_resized, dark_image=img_resized, size=(300, 300))

        label = ctk.CTkLabel(self.chat_frame, image=ctk_img, text="")
        label.image = ctk_img
        label.pack(anchor="w", padx=10, pady=4)

        # Add right-click menu
        label.bind("<Button-1>", lambda e: self.show_image_menu(e, image_obj, path))

        self.image_refs.append(ctk_img)
        self.loaded_images.append((image_obj, path))

    def show_image_menu(self, event, image, path):
        menu = Menu(self, tearoff=0)
        menu.add_command(label="💾 Save Image", command=lambda: self.ask_to_save_image(image))
        menu.add_command(label="✏️ Edit Image", command=lambda: self.edit_image(image))
        menu.add_command(label="🔁 Resend Image", command=lambda: self.display_image(image, path))
        try:
            menu.tk_popup(event.x_root, event.y_root)
        finally:
            menu.grab_release()

    def show_text_menu(self, event, label, text):
        menu = Menu(self, tearoff=0)
        menu.add_command(label="📝 Edit Text", command=lambda: self.edit_text(label, text))
        menu.add_command(label="💾 Save Text", command=lambda: self.save_text_to_file(text))
        menu.add_command(label="🔁 Resend Text", command=lambda: self.display_message("You", text.split(":", 1)[1].strip()))
        try:
            menu.tk_popup(event.x_root, event.y_root)
        finally:
            menu.grab_release()

    def edit_text(self, label, old_text):
        new_text = simpledialog.askstring("Edit Message", "Edit your message:", initialvalue=old_text.split(":", 1)[1].strip())
        if new_text:
            new_label_text = f"{old_text.split(':', 1)[0]}: {new_text}"
            label.configure(text=new_label_text)

    def save_text_to_file(self, text):
        save_path = filedialog.asksaveasfilename(
            defaultextension=".txt",
            filetypes=[("Text files", "*.txt")],
            title="Save Message"
        )
        if save_path:
            try:
                with open(save_path, 'w') as f:
                    f.write(text)
                self.display_message("System", f"Text saved to: {save_path}")
            except Exception as e:
                self.display_message("System", f"Failed to save text: {e}")

    def ask_to_save_image(self, image_obj):
        save_path = filedialog.asksaveasfilename(
            defaultextension=".png",
            filetypes=[("PNG files", "*.png"), ("JPEG files", "*.jpg"), ("All files", "*.*")],
            title="Save Image As..."
        )
        if save_path:
            try:
                image_obj.save(save_path)
                self.display_message("System", f"Image saved to: {save_path}")
            except Exception as e:
                self.display_message("System", f"Failed to save image: {e}")

    def edit_image(self, img):
        editor = Toplevel(self)
        editor.title("Image Editor")
        editor.geometry("400x400")

        img_resized = img.resize((250, 250))
        img_tk = ImageTk.PhotoImage(img_resized)
        img_label = ctk.CTkLabel(editor, text="", image=img_tk)
        img_label.image = img_tk
        img_label.pack(pady=10)

        btn_frame = ctk.CTkFrame(editor)
        btn_frame.pack(pady=10)

        def apply_grayscale():
            gray_img = ImageOps.grayscale(img)
            self.display_image(gray_img)

        def apply_rotate():
            rotated_img = img.rotate(90)
            self.display_image(rotated_img)

        ctk.CTkButton(btn_frame, text="Grayscale", command=apply_grayscale).grid(row=0, column=0, padx=5)
        ctk.CTkButton(btn_frame, text="Rotate", command=apply_rotate).grid(row=0, column=1, padx=5)

        editor.mainloop()

class App:
    def __init__(self):
        option = sys.argv
        if len(option) > 1:
            if option[1] == '--gui' or option[1] == '-g':
                GUI()

            if option[1] == '--web' or option[1] == '-w':
                Web()

            if option[1] == '--help' or option[1] == '-h':
                print('<======== Help ========>\nOpen With GUI : [--gui, -g]\nOpen With Website : [--web, -w]\nHelp : [--help, -h]\n\n')

        choice = input("<======= Choice =======>\n1. Open With GUI.\n2. Open With Website\n\n>>> ")

        if choice.isdigit():
            if choice == '1':
                GUI()

            elif choice == '2':
                import PYAS

            else:
                print(f'Invalid option {choice}')

        else:
            if choice.lower() == 'open with gui':
                GUI()

            elif choice.lower() == 'open with website' or choice.lower() == 'open with web':
                Web()

            else:
                print(f'Invalid option {choice}')

if __name__ == '__main__':
    App()



import cv2
import os
import importlib.resources
from ultralytics import YOLO

def yolo_detect_objects(frame, model, target_class="person"):
    """
    Runs YOLOv8 detection on a frame.
    Returns detected classes list and the annotated frame.
    """
    results = model(frame)[0]  # Run detection on frame

    # Extract detected class names from results
    detected_classes = [model.names[int(box.cls[0])] for box in results.boxes]

    # Annotated frame with bounding boxes
    annotated_frame = results.plot()

    if target_class in detected_classes:
        return detected_classes, annotated_frame
    else:
        return [], annotated_frame  # No target_class detected

def EYE():
    try:
        # Try loading model using importlib.resources (Python 3.9+)
        path = importlib.resources.files(__package__) / "yolov8n.pt"
        model = YOLO(str(path))
    except Exception:
        # Fallback: load from current script directory
        here = os.path.dirname(os.path.abspath(__file__), "eye", "yolov8n.pt")
        model_path = os.path.join(here, "yolov8n.pt")
        model = YOLO(model_path)

    cap = cv2.VideoCapture(0)
    if not cap.isOpened():
        print("Error: Could not open webcam")
        return []

    detected_objects = []

    while True:
        ret, frame = cap.read()
        if not ret:
            break

        detected_objects, annotated_frame = yolo_detect_objects(frame, model, target_class="person")

        cv2.imshow("YOLOv8 Detection", annotated_frame)

        # Exit loop if person detected or 'q' pressed
        if "person" in detected_objects or cv2.waitKey(1) & 0xFF == ord('q'):
            break

    cap.release()
    cv2.destroyAllWindows()
    return detected_objects



import torch
import torch.nn as nn
import torchaudio
from torch.utils.data import Dataset, DataLoader
import sounddevice as sd
import numpy as np
from scipy.io.wavfile import write
import os

characters = 'abcdefghijklmnopqrstuvwxyz'

num_classes = len(characters) + 1

char_to_idx = {c: i for i, c in enumerate(characters, 1)}

idx_to_char = {i: c for c, i in char_to_idx.items()}

def text_to_int(text) -> list:
    return [char_to_idx.get(c, 0) for c in text.lower()]

def int_to_text(indices) -> str:
    return ''.join(idx_to_char[i] for i in indices if i != 0)

class SpeechDataset(Dataset):
    def __init__(self, audio_files, transcripts) -> None:
        self.audio_files = audio_files
        self.transcripts = transcripts
        self.transform = torchaudio.transforms.MelSpectrogram(sample_rate= 16000, n_mels=128)

    def __len__(self):
        return len(self.audio_files)

    def __getitem__(self, idx):
        waveform, sample_rate = torchaudio.load(self.audio_files[idx])

        if sample_rate != 16000:
            resampler = torchaudio.transforms.Resample(sample_rate, 16000)
            waveform = resampler(waveform)

        features = self.transform(waveform).squeeze(0).transpose(0, 1)
        label = torch.tensor(text_to_int(self.transcripts[idx]))

        return features, label

def collect_fn(batch):
    features = [item[0] for item in batch]
    label = [item[1] for item in batch]
    feature_lengths = torch.tensor([f.size(0) for f in features])
    label_lengths = torch.tensor([len(l) for l in label])
    padded_features = nn.utils.rnn.pad_sequence(features, batch_first= True).transpose(1, 2)
    padded_labels = torch.cat(label)

    return padded_features, padded_labels, feature_lengths, label_lengths

class SpeechRecognitionModel(nn.Module):
    def __init__(self, input_dim: int = 128, hidden_dim: int = 256, output_dim: int = num_classes, num_layers: int = 3) -> None:
        super().__init__()
        self.rnn = nn.GRU(input_dim, hidden_dim, num_layers, batch_first= True, bidirectional = True)
        self.fc = nn.Linear(hidden_dim * 2, output_dim)

    def forward(self, x):
        x = x.transpose(1, 2)
        x, _ = self.rnn(x)
        return x.log_softmax(dim= -1)

model = SpeechRecognitionModel()
ctc_loss = nn.CTCLoss(blank = 0, zero_infinity= True)
optimizer = torch.optim.Adam(model.parameters(), lr= 0.001)

print('Traning Started!')
def train(dataloder):
    model.train()
    for features, labels, feature_lengths, label_lengths in dataloder:
        optimizer.zero_grad()
        outputs = model(features)
        outputs = outputs.permute(1, 0, 2)
        loss = ctc_loss(outputs, labels, feature_lengths, label_lengths)
        loss.backward()
        optimizer.step()
        #print(f'Loss: {loss.item()}')

print('Traning Completed!')

def greedy_decode(output, input_lengths):
    decoded_batch = []
    for i, length in enumerate(input_lengths):
        probs = output[i][:length]
        preds = torch.argmax(probs, dim= -1).tolist()
        prev = -1
        decoded = [p for p in preds if p != prev and p != 0]
        prev = preds[0] if preds else -1
        decoded_batch.append(int_to_text(decoded))

    return decoded_batch

def record(time_duration: int = 5, sample_rate: int = 16000) -> str:
    audio = sd.rec(int(time_duration * sample_rate), samplerate= sample_rate, channels= 1, dtype= 'int16')
    sd.wait()
    file_name = r'.\assert\rec_audio.wav'
    write(file_name, sample_rate, audio)
    return 'DONE'


class STTModel:
    def __init__(self, time_duration: int = 5, sample_rate: int = 16000):
        self.duration = time_duration # seconds of recording
        self.sample_rate = sample_rate # 16KHz is common for speech recognition

    def predict(self) -> str:
        record()
        waveform, _ = torchaudio.load(r'.\assert\rec_audio.wav')
        features = torchaudio.transforms.MelSpectrogram(self.sample_rate, n_mels= 128)
        features = features.squeeze(0).transpose(0, 1).unsqueeze(0).transpose(1, 2)

        with torch.no_grad():
            out = model(features)
            decoded = greedy_decode(out.transpose(0, 1), [out.size(1)])
            return decoded[0]

    def save(self):
        torch.save(model.state_dict(), r'.\stt_model.pth')

    def load(self):
        if os.path.exists(r'.\stt_model.pth'):
            model.load_state_dict(torch.load(r".\stt_model.pth"))
            model.eval()

sttModel = STTModel()
if os.path.exists(r'.\stt_model.wav'):
    pass

else:
    audio_files = [r'.\assert\audio1.wav', r'.\assert\audio2.wav']
    transcripts = ['Hello world', 'testing']

    dataset = SpeechDataset(audio_files, transcripts)
    dataloder = DataLoader(dataset, batch_size= 2, collate_fn= collect_fn)
    train(dataloder)
    sttModel.save()

class STT:
    def __init__(self) -> None:
        sttModel.load()
        self.text = sttModel.predict()

    def outputs(self) -> str:
        return self.text



from .core import Brain, AdvanceBrain
from .TTS import TTS
from .TTS import TTS as text
#from .STT import STT, STTModel
from .Camera import Camera
from .Camera import Start
from .Context import Contexts as contexts
from .Context import Contexts
from .Memory import Memory
from .Memory import Memory as memory
#from .TTI import TTI
import nltk

if input('Want to download nltk data (y/n) : ').lower()[0] == "y":
    nltk.download('punkt', quiet=True)
    nltk.download('stopwords', quiet=True)
    nltk.download('averaged_perceptron_tagger', quiet=True)
    nltk.download('wordnet', quiet=True)
    nltk.download('omw-1.4', quiet=True)
    nltk.download('maxent_ne_chunker', quiet=True)
    nltk.download('words', quiet=True)
    nltk.download('treebank', quiet=True)
    nltk.download('brown', quiet=True)
    nltk.download('conll2000', quiet=True)
    nltk.download('movie_reviews', quiet=True)
    nltk.download('names', quiet=True)
    nltk.download('gazetteers', quiet=True)
    nltk.download('tagsets', quiet=True)
    nltk.download('help/tagsets', quiet=True)

__doc__ = """
PythonAI Brain
Make your first AI Assistant in python. No complex setup, No advance coding. Just install configure and run!

Installation
Install pythonaibrain package.

pip install pythonaibrain==1.0.7
Modules
Camera
TTS
PTT
Memory
Context
Brain
Advance Brain
Camera
PyAI supports Camera to click photos and make videos, it can save photos or videos and also send Images and Videos to PyAI to take answer

Example
For start your camera

# Import modules
import pyai
from pyai import Camera
import tkinter as tk
from tkinter import *

root = tk.Tk() # Create the GUI
Camera(root) # Call the Function and pass the master as root
root.mainloop() # Start GUI app
From this you can easly use camera in your program.

Or,

from pyai.Camera import Start
Start()

Or,

from pyai import Brain

brain = Brain()
brain.process_messages('Click Photo')

TTS
TTS stands for Text To Speech, it convert text into both Male voice and Female voice.

Example
# Import modules
import pyai
from pyai import TTS

tts = TTS(text = 'Hello World')
tts.say(voice= 'Male') # for male voice
tts.say(voice= 'Female') #for female voice
tts.say() -> By default it takes Male voice tts.say(voice= 'Male') -> Pass the voice as Male tts.say(voice= 'Female') -> Pass the voice as Female


PTT
PTT stands for PDF To Text, it can extract text from a given image

Example
# Import modules
import pyai
from pyai import PTT

itt = ITT(path = 'example.jpeg') # You can change example.jpeg from your file name
text = itt.outputs() # It returns the text extract from the given image
Syntax
PTT(path: str = None)
Give your own file path.

Context
It is a module in pyai which can able to extract answers from the give context

Example
# Import modules
import pyai
from pyai import Contexts

context = '''
Patanjali Ayurved is an Indian company. It was founded by Baba Ramdev and Acharya Balkrishna in 2006.
'''

question = 'Who founded Patanjali Ayurved?'
contexts = Contexts()
answer = contexts.ask(context= context, question= question)
Or, Also

# Import modules
import pyai
from pyai import Contexts as contexts

context = '''
Patanjali Ayurved is an Indian company. It was founded by Baba Ramdev and Acharya Balkrishna in 2006.
'''

question = 'Who founded Patanjali Ayurved?'
answer = contexts.ask(context= context, question= question)
Brain
It's a simple brain module which configure the input message.

What it does
It classify the input message and find the type of message, like

Question
Answer
Command
Shutdown
Make Directory
Statement
Name
Know
Start
It also extract the name, location, and age from the given message, by using NER.

Question
The Brain Module classify the given message weather it is a question or something else if answer then returns Question.

Answer
The Brain Module classify the given message weather it is a answer or something else if answer then returns Answer.

Command
The Brain Module classify the given message weather it is a command or something else if command then returns Command

Shutdown
The Brain Module also classify the given message weather the given command shutdown or not if it is then it shutdown your device and it returns Shutdown

But there are few issue releated to it :

This command doesn't support website to run this command, because it need a terminal support.
This doesn't run or work on Android, IOS.
Make Directory
The Brain Module also classify the given message weather the given command Make Directory or not if it is then it create a Directory on your device and returns Make Dir.

It generally comes under File handling of the PyAI Module which is also known as fh.

Statement
The Brain Module also classify the given message weather the given command statement or not, if it is then it statement then it returns Statement.

Statement -> It means a simple text which is not a question, answer, command, etc... It a simple text. Like for example:

The sun rises in the east.
Name
The Brain Module also classify the given message weather the given command name or not, if it is then it name then it returns Name.

Name -> It means the input message is caring name or specify the name like

I'm Divyanshu.

Myself Divyanshu.

Divyanshu Sinha
Know
Know is similar to Statement.

Do you know ___ ?
Like that.

Start
The Brain Module also classify the given message weather the given command start or not, if it is then it start any thing on your device and it returns Start.

Like:

Start www.youtube.com
Or,

Start Notepad
But it have issue:

It doesn't works with website, because it support terminals like CMD for Window and etc.
It also doesn't works in Android, IOS.
How to create intents.json
You should know how to create a intents.json for run this Brain Module.

Pattern to create a intents.json file:
{
    "intents":[{
        "tag": "greeting",
      "patterns": ["Hi", "Hello", "Hey", "What's up?", "Howdy", "Greetings", "Hi there", "Is anyone there?", "Yo!"],
      "responses": ["Hello! How can I help you today?", "Hey there!", "Hi! What can I do for you?"]
    },
    {
        "tag": "bye",
        "patterns": ["By", "See you soon", "See u soon", "Take care"],
        "responces":["Bye! have a greate day", "See you"]
    },
    ]
}
From this way you can create your own database.

Remember this Database file in .json

How to use Brain Module
To use Brain Module we should import Brain from PyAI

from pyai import Brain
After importing the Module we use it in our main program as,

brain = Brain(intents_path= 'intents.json') # Use can replace intents.json with you database file name but extention should be same (.json). Or u can also use Brain() it can also work
After this, we predict the message type of we can say classify the message

message = input('Message : ')
message_type = brain.predict_message_type(message= message) # On using predict_message_type() function we get the type of message is (question, answer, statement, command, shutdown, make directory, name, know, etc...)
By gating the message type, we find the perfect answer for the message

if message_type in ['Question', 'Answer']:
    print(f'Answer : {brain.process_messages(message = message)}')

From these things you can create your own AI Assistant. But this is basic.

For Advance we can use Advance Brain Module.

Advance Brain
This is advance version of Brain Module It work like Brain but smartly.

What it does
It classify the input message and find the type of message, like

Question
Answer
Command
Shutdown
Make Directory
Statement
Name
Know
Start
It also extract the name, location, and age from the given message, by using NER.

Question
The Brain Module classify the given message weather it is a question or something else if answer then returns Question.

Answer
The Brain Module classify the given message weather it is a answer or something else if answer then returns Answer.

Command
The Brain Module classify the given message weather it is a command or something else if command then returns Command

Shutdown
The Brain Module also classify the given message weather the given command shutdown or not if it is then it shutdown your device and it returns Shutdown

But there are few issue releated to it :

This command doesn't support website to run this command, because it need a terminal support.
This doesn't run or work on Android, IOS.
Make Directory
The Brain Module also classify the given message weather the given command Make Directory or not if it is then it create a Directory on your device and returns Make Dir.

It generally comes under File handling of the PyAI Module which is also known as fh.

Statement
The Brain Module also classify the given message weather the given command statement or not, if it is then it statement then it returns Statement.

Statement -> It means a simple text which is not a question, answer, command, etc... It a simple text. Like for example:

The sun rises in the east.
Name
The Brain Module also classify the given message weather the given command name or not, if it is then it name then it returns Name.

Name -> It means the input message is caring name or specify the name like

I'm Divyanshu.

Myself Divyanshu.

Divyanshu Sinha
Know
Know is similar to Statement.

Do you know ___ ?
Like that.

Start
"The Brain Module classifies the given message whether the given command start or not, if it is then it start any thing on your device and it returns Start.

Like:

Start www.youtube.com
Or,

Start Notepad
But it have issue:

It doesn't works with website, because it support terminals like CMD for Window and etc.
It also doesn't works in Android, IOS.
How to create intents.json
You should know how to create a intents.json for run this Advance Brain Module.

Pattern to create a intents.json file:
{
    "intents":[{
        "tag": "greeting",
      "patterns": ["Hi", "Hello", "Hey", "What's up?", "Howdy", "Greetings", "Hi there", "Is anyone there?", "Yo!"],
      "responses": ["Hello! How can I help you today?", "Hey there!", "Hi! What can I do for you?"]
    },
    {
        "tag": "bye",
        "patterns": ["By", "See you soon", "See u soon", "Take care"],
        "responce":["Bye! have a greate day", "See you"]
    },
    ]
}
From this way you can create your own database.

Remember this Database file in .json

How to use Advance Brain Module
To use Advance Brain Module we should import Brain from PyAI

from pyai import AdvanceBrain
After importing the Module we use it in our main program as,

advance_brain = AdvanceBrain(intents_path= 'intents.json') # Use can replace intents.json with you database file name but extention should be same (.json). Or, can also do Advancebrain() It also works.
After this, we predict the message type of we can say classify the message

message = input('Message : ')
message_type = advance_brain.predict_message_type(message= message) # On using predict_message_type() function we get the type of message is (question, answer, statement, command, shutdown, make directory, name, know, etc...)
By gating the message type, we find the perfect answer for the message

if message_type in ['Question', 'Answer']:
    print(f'Answer : {advance_brain.process_messages(message = message)}')

Python AI Modules and their use
| Module Name | Description | | :---: | :---:| |Brain| It is use to create Brain for AI by passing .json file (or, Knowledge for Brain)| |AdvanceBrain|It is use to create Advance Brain for AI by passing .json file (or, Knowledge for Brain). It can understand better than Brain| |TTS|Convert text into Voice| |STT|Convert Voice into Text| |TTI|Convert Text into Image| |ITT|Convert Or, extract Image into Text| |Camera|Use camera to click photos and make videos| |Context|Get Answer from the context for the respective question|

PythonAI Brain also provides built-in AI Assistant
If you don't want to create your own AI assistant by coding or you want to see how this modules work you can also use PyBrain which is a built-in python AI assistance, provided by pythonaibrain == 1.0.2

How to use it
import PyBrain
PyBrain('-g')
By using this you can use PyBrain in GUI.

Or,

import PyBrain
PyBrain('-w')
By using this you can use PyBrain in Web.

Or,

import PyBrain
PyBrain('-h')
By using this Help panel open.

Visit PyPI for installation."""

__version__ = "1.0.6"
__author__ = "Divyanshu Sinha"
__all__ = [
    "Brain",
    "AdvanceBrain",
    "TTS",
    "PTT",
    "Camera",
    "Contexts",
    "Memory",
]



import win32com.client
import pyttsx3
import sys

voice_list = ['microsoft david desktop', 'microsoft zira desktop', 'microsoft mark desktop']

class TTS:
    def __init__(self, text: str | None = 'Hello from Py AI') -> None:
        self.text = text

    def OsType(self) -> str:
        return sys.platform

    def say(self, voice: str | None = 'Male', **name) -> None:
        if voice.lower() == 'male':
            index = 0

        elif voice.lower() == 'female':
            index = 1

        else:
            print(f'Invalid voice name : {voice}')
            index = 0

        engine = pyttsx3.init()
        engine.setProperty('rate', 150)
        engine.setProperty('volume', 1.0)

        voices = engine.getProperty('voices')
        engine.setProperty('voice', voices[index].id)

        engine.say(self.text)
        engine.runAndWait()

text = ''
TTS(text).say()



import os
import json
import torch
import torch.nn as nn
from torch.utils.data import Dataset, DataLoader
from torchvision import transforms
from PIL import Image
import matplotlib.pyplot as plt
import random


# Tokenizer for text
class SimpleTokenizer:
    def __init__(self):
        self.word2idx = {"<PAD>": 0}
        self.idx2word = {0: "<PAD>"}
        self.idx = 1

    def encode(self, text):
        tokens = text.lower().split()
        indices = []
        for token in tokens:
            if token not in self.word2idx:
                self.word2idx[token] = self.idx
                self.idx2word[self.idx] = token
                self.idx += 1
            indices.append(self.word2idx[token])
        return torch.tensor(indices, dtype=torch.long)

    def vocab_size(self):
        return len(self.word2idx)


# Custom Dataset Loader from structured folders and JSON prompts
class TextImageDataset(Dataset):
    def __init__(self, data_dir, json_file, tokenizer, transform=None):
        self.data_dir = data_dir
        self.transform = transform
        self.tokenizer = tokenizer
        with open(json_file, "r") as f:
            self.annotations = json.load(f)
        self.samples = list(self.annotations.items())

    def __len__(self):
        return len(self.samples)

    def __getitem__(self, idx):
        image_file, prompt = self.samples[idx]
        image_path = os.path.join(self.data_dir, image_file)
        image = Image.open(image_path).convert("RGB")
        if self.transform:
            image = self.transform(image)
        text = self.tokenizer.encode(prompt)
        return text, image


# Text Encoder
class TextEncoder(nn.Module):
    def __init__(self, vocab_size, embed_dim=256):
        super(TextEncoder, self).__init__()
        self.embedding = nn.Embedding(vocab_size, embed_dim)
        self.rnn = nn.GRU(embed_dim, embed_dim, batch_first=True)

    def forward(self, x):
        embedded = self.embedding(x)
        _, hidden = self.rnn(embedded)
        return hidden.squeeze(0)


# Image Generator
class ImageGenerator(nn.Module):
    def __init__(self, embedding_dim=256):
        super(ImageGenerator, self).__init__()
        self.net = nn.Sequential(
            nn.Linear(embedding_dim, 512 * 4 * 4),
            nn.ReLU(True),
            nn.Unflatten(1, (512, 4, 4)),
            nn.ConvTranspose2d(512, 256, 4, 2, 1),  # 8x8
            nn.ReLU(True),
            nn.ConvTranspose2d(256, 128, 4, 2, 1),  # 16x16
            nn.ReLU(True),
            nn.ConvTranspose2d(128, 64, 4, 2, 1),   # 32x32
            nn.ReLU(True),
            nn.ConvTranspose2d(64, 3, 4, 2, 1),     # 64x64
            nn.Tanh()
        )

    def forward(self, embedding):
        return self.net(embedding)


# Main Class for TTI (Text-To-Image)
class TextToImage:
    def __init__(self, data_dir, json_file):
        self.tokenizer = SimpleTokenizer()
        self.transform = transforms.Compose([
            transforms.Resize((64, 64)),
            transforms.ToTensor()
        ])
        self.dataset = TextImageDataset(data_dir, json_file, self.tokenizer, self.transform)
        self.dataloader = DataLoader(self.dataset, batch_size=4, shuffle=True)
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

        vocab_size = self.tokenizer.vocab_size()
        self.text_encoder = TextEncoder(vocab_size).to(self.device)
        self.generator = ImageGenerator().to(self.device)

        self.optimizer = torch.optim.Adam(list(self.text_encoder.parameters()) + list(self.generator.parameters()), lr=0.001)
        self.criterion = nn.MSELoss()

    def train(self, epochs=5):
        print("Starting training...")
        for epoch in range(epochs):
            total_loss = 0
            for texts, images in self.dataloader:
                texts = texts.to(self.device)
                images = images.to(self.device)

                text_embeddings = self.text_encoder(texts)
                generated_images = self.generator(text_embeddings)

                loss = self.criterion(generated_images, images)
                self.optimizer.zero_grad()
                loss.backward()
                self.optimizer.step()
                total_loss += loss.item()

            #print(f"Epoch {epoch+1}/{epochs}, Loss: {total_loss:.4f}")
            #self.save_output(generated_images[0].detach().cpu(), f"output_epoch_{epoch+1}.png")
        print(f'Traning Done!')

    def generate(self, prompt):
        self.text_encoder.eval()
        self.generator.eval()
        with torch.no_grad():
            encoded = self.tokenizer.encode(prompt).unsqueeze(0).to(self.device)
            embedding = self.text_encoder(encoded)
            image = self.generator(embedding)
            return image.squeeze(0).cpu()

    def save_output(self, tensor_img, filename):
        img = transforms.ToPILImage()(tensor_img.clamp(0, 1))
        img.save(filename)
        print(f"[✓] Saved image as {filename}")

    def display_image(self, image_tensor):
        img = transforms.ToPILImage()(image_tensor.clamp(0, 1))
        plt.imshow(img)
        plt.axis("off")
        plt.show()


class TTI:
    def __init__(self, text: str, data_dir: str = 'assert', json_file: str = "dataset.json") -> None:
        self.text = text
        self.tti = TextToImage(data_dir, json_file)
        self.tti.train(epochs=10)
        self.gen_img = self.tti.generate(self.text)

    def display(self) -> None:
        self.tti.display_image(self.gen_img)

    def save(self, folder_name: str = "output.png") -> None:
        self.tti.save_output(self.gen_img, f"{folder_name.split('.')[0]}.png")



import cv2
import tkinter as tk
from tkinter import Label, Button
from PIL import Image, ImageTk
from datetime import datetime
from pyzbar import pyzbar

class Camera:
    def __init__(self, window):
        self.data = set()
        self.window = window
        self.window.title("📷 Cool Camera App")
        self.window.configure(bg="#1e1e1e")
        self.window.resizable(False, False)

        self.recording = False
        self.out = None

        # Video capture object
        self.video_capture = cv2.VideoCapture(0)

        # GUI Components
        self.label = Label(window, bg="#1e1e1e")
        self.label.pack(padx=10, pady=10)

        self.capture_button = Button(
            window, text="📸 Capture Image", command=self.capture_image,
            font=("Segoe UI", 12, "bold"), fg="white", bg="#4CAF50",
            activebackground="#45a049", relief="flat", width=20
        )
        self.capture_button.pack(pady=5)

        self.record_button = Button(
            window, text="🎥 Start Recording", command=self.toggle_recording,
            font=("Segoe UI", 12, "bold"), fg="white", bg="#2196F3",
            activebackground="#1976D2", relief="flat", width=20
        )
        self.record_button.pack(pady=5)

        self.quit_button = Button(
            window, text="❌ Quit", command=self.quit_app,
            font=("Segoe UI", 12, "bold"), fg="white", bg="#f44336",
            activebackground="#e53935", relief="flat", width=20
        )
        self.quit_button.pack(pady=10)

        self.update_frame()

    def update_frame(self):
        ret, frame = self.video_capture.read()
        try:
            qr_codes=pyzbar.decode(frame)
            for qr_code in qr_codes:
                self.data.add(qr_code.data.decode('utf-8'))
        except:
            pass
        if ret:
            # Save frame if recording
            if self.recording and self.out:
                self.out.write(frame)

            # Convert BGR to RGB for display
            rgb_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
            img = Image.fromarray(rgb_frame)
            imgtk = ImageTk.PhotoImage(image=img)

            self.label.imgtk = imgtk
            self.label.configure(image=imgtk)

        self.window.after(10, self.update_frame)

    def capture_image(self):
        ret, frame = self.video_capture.read()
        if ret:
            filename = f"captured_{datetime.now().strftime('%Y%m%d_%H%M%S')}.png"
            cv2.imwrite(filename, frame)
            print(f"✅ Image saved as {filename}")

    def toggle_recording(self):
        if not self.recording:
            filename = f"output_{datetime.now().strftime('%Y%m%d_%H%M%S')}.avi"
            fourcc = cv2.VideoWriter_fourcc(*'XVID')
            fps = 20.0
            frame_size = (
                int(self.video_capture.get(3)),
                int(self.video_capture.get(4))
            )
            self.out = cv2.VideoWriter(filename, fourcc, fps, frame_size)
            self.recording = True
            self.record_button.config(text="⏹️ Stop Recording", bg="#FF5722")
            print(f"🎥 Recording started: {filename}")
        else:
            self.recording = False
            self.out.release()
            self.out = None
            self.record_button.config(text="🎥 Start Recording", bg="#2196F3")
            print("🛑 Recording stopped.")

    def quit_app(self):
        if self.recording and self.out:
            self.out.release()
        self.video_capture.release()
        self.window.destroy()

    def return_data(self) -> set:
        return self.data

def Start() -> set:
    # Start the GUI app
    root = tk.Tk()
    app = Camera(root)
    root.mainloop()
    return app.return_data()
